
/* Chrysalide - Outil d'analyse de fichiers binaires
 * logs.c - équivalent Python du fichier "gui/panels/logs.c"
 *
 * Copyright (C) 2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "logs.h"


#include <pygobject.h>


#include <core/logs.h>


#include "../helpers.h"



/* Fournit la verbosité des messages système. */
static PyObject *py_logs_get_verbosity(PyObject *, PyObject *);

/* Définit la verbosité des messages système. */
static PyObject *py_logs_set_verbosity(PyObject *, PyObject *);

/* Affiche un message dans le journal des messages système. */
static PyObject *py_logs_log_message(PyObject *, PyObject *);

/* Définit les constantes pour les types de message. */
static bool define_python_log_constants(PyTypeObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : self = classe assurant le lien avec l'éditeur de messages.   *
*                args = arguments fournis à l'appel.                          *
*                                                                             *
*  Description : Fournit la verbosité des messages système.                   *
*                                                                             *
*  Retour      : Plus faible niveau des types de message affichés.            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_logs_get_verbosity(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Conversion à retourner      */
    LogMessageType verbosity;               /* Niveau de filtre de message */

    verbosity = get_log_verbosity();

    result = PyLong_FromUnsignedLong(verbosity);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self = classe assurant le lien avec l'éditeur de messages.   *
*                args = arguments fournis à l'appel.                          *
*                                                                             *
*  Description : Définit la verbosité des messages système.                   *
*                                                                             *
*  Retour      : Rien en équivalent Python.                                   *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_logs_set_verbosity(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Bilan à retourner           */
    unsigned long verbosity;                /* Niveau de filtre de message */

    if (!PyArg_ParseTuple(args, "k", &verbosity))
        return NULL;

    set_log_verbosity(verbosity);

    result = Py_None;
    Py_INCREF(result);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self = classe assurant le lien avec l'éditeur de messages.   *
*                args = arguments fournis à l'appel.                          *
*                                                                             *
*  Description : Affiche un message dans le journal des messages système.     *
*                                                                             *
*  Retour      : Rien en équivalent Python.                                   *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_logs_log_message(PyObject *self, PyObject *args)
{
    PyObject *result;                       /* Bilan à retourner           */
    unsigned long type;                     /* Espèce du message           */
    const char *msg;                        /* Contenu du message          */

    if (!PyArg_ParseTuple(args, "ks", &type, &msg))
        return NULL;

    switch (type)
    {
        case LMT_INFO:
        case LMT_PROCESS:
        case LMT_WARNING:
        case LMT_ERROR:
        case LMT_BAD_BINARY:
            log_simple_message(type, msg);
            result = Py_None;
            Py_INCREF(result);
            break;

        default:
            PyErr_SetString(PyExc_ValueError,
                            _("Invalid type of message"));
            result = NULL;
            break;

    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_logs_type(void)
{
    static PyMethodDef py_logs_methods[] = {
        {
            "get_verbosity", (PyCFunction)py_logs_get_verbosity,
            METH_NOARGS | METH_STATIC,
            "get_verbosity(, /)\n--\n\nGet the log verbosity."
        },
        {
            "set_verbosity", (PyCFunction)py_logs_set_verbosity,
            METH_VARARGS | METH_STATIC,
            "set_verbosity(, /)\n--\n\nSet the log verbosity."
        },
        {
            "log_message", (PyCFunction)py_logs_log_message,
            METH_VARARGS | METH_STATIC,
            "log_message(type, msg, /)\n--\n\nDisplay a message in the log window, if any."
        },
        { NULL }

    };

    static PyGetSetDef py_logs_getseters[] = {
        { NULL }
    };

    static PyTypeObject py_logs_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name = "pychrysalide.core.logs",
        .tp_basicsize = sizeof(PyObject) + 80,

        .tp_flags = Py_TPFLAGS_DEFAULT,

        .tp_doc = "Python object for logs",

        .tp_methods = py_logs_methods,
        .tp_getset  = py_logs_getseters

    };

    return &py_logs_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : obj_type = type dont le dictionnaire est à compléter.        *
*                                                                             *
*  Description : Définit les constantes pour les types de message.            *
*                                                                             *
*  Retour      : true en cas de succès de l'opération, false sinon.           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static bool define_python_log_constants(PyTypeObject *obj_type)
{
    bool result;                            /* Bilan à retourner           */

    result = true;

    result &= PyDict_AddIntMacro(obj_type, LMT_INFO);
    result &= PyDict_AddIntMacro(obj_type, LMT_PROCESS);
    result &= PyDict_AddIntMacro(obj_type, LMT_WARNING);
    result &= PyDict_AddIntMacro(obj_type, LMT_ERROR);
    result &= PyDict_AddIntMacro(obj_type, LMT_BAD_BINARY);
    result &= PyDict_AddIntMacro(obj_type, LMT_COUNT);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.gui.panels.LogPanel'.  *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool register_python_logs(PyObject *module)
{
    PyTypeObject *py_logs_type;             /* Type Python pour 'logs'     */
    int ret;                                /* Bilan d'un appel            */

    py_logs_type = get_python_logs_type();

    py_logs_type->tp_new = PyType_GenericNew;

    if (PyType_Ready(py_logs_type) != 0)
        return false;

    if (!define_python_log_constants(py_logs_type))
        return false;

    Py_INCREF(py_logs_type);
    ret = PyModule_AddObject(module, "logs", (PyObject *)py_logs_type);

    return (ret == 0);

}
